/**************************************************************************************************

Copyright (c) 2007   Tsunehiko Kato  and  4D2U Project, National Astronomical Observatory of Japan

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

ȉɒ߂ɏ]A{\tgEFAъ֘Ãt@Ciȉu\tgEFAvj̕擾
邷ׂĂ̐lɑ΂A\tgEFA𖳐ɈƂ𖳏ŋ܂Bɂ́A\tgEFA̕
gpAʁAύXAAfځAЕzATuCZXA/܂͔̔錠Aу\tgEFA
񋟂鑊ɓƂ錠Ɋ܂܂܂B

L̒쌠\і{\A\tgEFÂׂĂ̕܂͏dvȕɋLڂ̂Ƃ
B

\tgEFÁû܂܁vŁAł邩Öقł邩킸A̕ۏ؂Ȃ񋟂܂B
łۏ؂Ƃ́AiA̖ړIւ̓KAьNQɂĂ̕ۏ؂܂݂܂AɌ
̂ł͂܂B ҂܂͒쌠҂́A_sׁAs@sׁA܂͂ȊOł낤ƁA\t
gEFAɋN܂͊֘AA邢̓\tgEFA̎gp܂͂̑̈ɂĐ؂̐A
QȂ̋`ɂĉ̐ӔCȂ̂Ƃ܂B
***************************************************************************************************/
#ifndef  __MTK_MENU_H__
#define  __MTK_MENU_H__
#include  <string.h>
#include  <typeinfo.h>
#include  <vector>
#include  <assert.h>

using namespace std;

//-------------------
//  j[ڂ̎
//-------------------
enum menu_item_type {
	MENU_TY_BASE,
	MENU_TY_CHILD,
	MENU_TY_COMMAND,
	MENU_TY_CHECK,
	MENU_TY_FVALUE
};


//------------------------
//  j[ڂ̊{NX
//------------------------
class menu;

class menu_item_base
{
	char nameJ[256];
	char nameE[256];

public:
	menu_item_base()										{}
	menu_item_base(const char* nameJ, const char* nameE)	{ set_name(nameJ, nameE); }
	~menu_item_base()										{} 

	void	set_name(const char* _nameJ, const char* _nameE) {
		strncpy(nameJ, _nameJ, 255);
		strncpy(nameE, _nameE, 255);
	}

	virtual	menu_item_type	get_type() const = 0;
	const char*				get_name_J() const				{ return nameJ; }
	const char*				get_name_E() const				{ return nameE; }
};


//------------
//  qj[
//------------
class menu_item_child  :  public menu_item_base
{
	menu* pChildMenu;
public:
	menu_item_child(const char* nameJ, const char* nameE, menu* pMenu) : menu_item_base(nameJ, nameE), pChildMenu(pMenu)  {};

	menu_item_type	get_type() const					{ return MENU_TY_CHILD; }
	void			set_child_menu(menu* pMenu)			{ pChildMenu = pMenu; }
	menu*			get_child_menu()					{ return pChildMenu; }
};


//-----------
//  R}h
//-----------
class menu_item_command  :  public menu_item_base
{
	int command_ID;
	int	idx;

public:
	menu_item_command(const char* nameJ, const char* nameE, int cmdID, int _idx) : menu_item_base(nameJ, nameE), command_ID(cmdID), idx(_idx)  {};

	menu_item_type	get_type() const			{ return MENU_TY_COMMAND; }
	void			set_ID(int ID)				{ command_ID = ID; }
	int				get_ID() const				{ return command_ID; }
	void			set_idx(int _idx)			{ idx = _idx; }
	int				get_idx() const				{ return idx; }
};


//----------
//  `FbN
//----------
class menu_item_check  :  public menu_item_base
{
	bool	bCheck;
	int		check_ID;
	int		idx;

public:
	menu_item_check(const char *nameJ, const char* nameE, bool bChk, int ID, int _idx) : menu_item_base(nameJ, nameE), bCheck(bChk), check_ID(ID), idx(_idx)  {};

	menu_item_type	get_type() const				{ return MENU_TY_CHECK; }
	void			set_check(bool bChk)			{ bCheck = bChk; }
	bool			get_check() const				{ return bCheck; }
	void			set_ID(int ID)					{ check_ID = ID; }
	int				get_ID() const					{ return check_ID; }
	void			set_idx(int _idx)				{ idx = _idx; }
	int				get_idx() const					{ return idx; }
};


//--------------
//  _l
//--------------
class menu_item_fvalue  :  public menu_item_base
{
	float	value;
public:
	menu_item_fvalue(const char* nameJ, const char* nameE, const float& val)  :  menu_item_base(nameJ, nameE), value(val)  {};

	menu_item_type	get_type() const				{ return MENU_TY_FVALUE; }
	void			set_value(const float& val)		{ value = val; }
	float			get_value() const				{ return value; }
};


//--------------
//  class menu
//--------------
class menu  :  public  std::vector<menu_item_base*>
{
	menu* pParentMenu;
	char titleJ[256], titleE[256];
	int current_pos;
public:
	menu() : current_pos(0)  {};
	~menu()	{ free(); }
	void	free();

	//---
	void	add_child_menu(menu* pChildMenu, const char* nameJ, const char* nameE)	{
		this->push_back(new menu_item_child(nameJ, nameE, pChildMenu) );
	}
	void	add_command(int command_id, int idx, const char* nameJ, const char* nameE) {
		this->push_back( new menu_item_command(nameJ, nameE, command_id, idx) );
	}
	void	add_check(int check_id, const char* nameJ, const char* nameE, int idx, bool bInitCheck) {
		this->push_back( new menu_item_check(nameJ, nameE, bInitCheck, check_id, idx) );
	}

	void	set_current_pos(int pos)	{ current_pos = pos; assert(pos>=0); }
	int		get_current_pos() const		{ return current_pos; }
	virtual void set_current_pos_auto()		{ set_current_pos(0); }

	void	set_title(const char* nameJ, const char* nameE)	{
		strncpy(titleJ, nameJ, 255);
		strncpy(titleE, nameE, 255);
	}

	const char* get_titleJ() const			{ return titleJ; }
	const char* get_titleE() const			{ return titleE; }
	void	set_check(int idx, bool bChk);
};


//---------------------
//  class menu_option
//---------------------
class menu_option  :  public menu
{
	int  check_pos;

public:
	menu_option() : check_pos(0)  {};
	menu_option(int idx) : check_pos(idx)  {};
	~menu_option() { menu::free(); }

	void set_current_pos_auto()	{
		assert(check_pos >= 0);
		set_current_pos(check_pos);
	}

	void	set_check(int idx)
		{
			assert(idx >= 0);
			check_pos = idx;
			int i=0;
			for (vector<menu_item_base*>::iterator it=this->begin(); it!=this->end(); ++it, ++i) {
				menu_item_check*  mi = dynamic_cast<menu_item_check*>(*it);
				if (mi != 0) {
					mi->set_check(i==idx);
				}
			}
		}

	void	set_check_by_index(int idx)
		{
			assert(idx >= 0);
			check_pos = 0;
			int i = 0;
			for (vector<menu_item_base*>::iterator it=this->begin(); it!=this->end(); ++it, ++i) {
				menu_item_check*  mi = dynamic_cast<menu_item_check*>(*it);
				if (mi != 0) {
					int  miidx = mi->get_idx();
					mi->set_check( miidx==idx );
					if (miidx == idx) {
						check_pos = i;
					}
				}
			}
		}


	int		get_check() const	{ return check_pos; }

	void	reset_check()	{
		for (vector<menu_item_base*>::iterator it=this->begin(); it!=this->end(); ++it) {
			menu_item_check*  mi = dynamic_cast<menu_item_check*>(*it);
			mi->set_check(false);
		}
	}
};


//------------------
//  menu selection
//------------------
class  menu_selection  :  public  std::vector<menu*>
{};



#endif
